# Copyright 2022 NetApp, Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from manilaclient.tests.functional.osc import base


class ShareNetworkSubnetsCLITest(base.OSCClientTestBase):

    def test_openstack_share_network_create_check(self):
        share_network = self.create_share_network()

        check_result = self.check_create_network_subnet(
            share_network['id'])

        self.assertEqual('True', check_result['compatible'])
        self.assertEqual('{}', check_result['hosts_check_result'])

    def test_openstack_share_network_create_check_restart(self):
        share_network = self.create_share_network()

        check_result = self.check_create_network_subnet(
            share_network['id'], restart_check=True)

        self.assertEqual('True', check_result['compatible'])
        self.assertEqual('{}', check_result['hosts_check_result'])
